@@ -1,7 +1,5 @@
 import git, os, subprocess, traceback
-
-def repeat_owners_permission(path):
-    subprocess.call('chmod -R a+u %s' % path, shell=True)
+import _common as common
 
 class _Git(object):
     """
@@ -68,11 +66,11 @@ class _Git(object):
         for fname, content in self.DEFAULT_FILES.items():
             path = os.path.join(self.gdir, fname)
             open(path, 'w').write(content)
-        repeat_owners_permission(self.dot_gdir)
+        common.repeat_owners_permission(self.dot_gdir)
             
     def open_repo(self):
         self.repo = self._open_repo()
-        repeat_owners_permission(self.dot_gdir)
+        common.repeat_owners_permission(self.dot_gdir)
     
     def _open_repo(self):
         return git.Repo(self.gdir)
