@@ -1,4 +1,5 @@
 import jinja2, os, re, httplib, traceback, json, sys, StringIO
+from datetime import datetime
 import DonkeySimple
 import DonkeySimple.DS as ds
 from DonkeySimple.DS import get_settings
@@ -13,8 +14,6 @@ from werkzeug.utils import escape
 from werkzeug.wsgi import SharedDataMiddleware
 # from werkzeug.exceptions import HTTPException, NotFound, BadRequest
 
-
-DEBUG_PORT = 4000
 class SERVER_MODES:
     UNKNOWN = 0
     DEBUG = 1
@@ -31,16 +30,17 @@ def make_dev_app():
     files as well as the site itself.
     SiteServeMiddleware takes care of some server like file serving.
     """
-    static_files= {'/favicon.ico': os.path.join('static', 'favicon.png'), 
-                   '/static': 'static', 
+    static_dir = os.path.join(os.path.dirname(__file__), 'static')
+    static_files= {'/favicon.ico': os.path.join(static_dir, 'favicon.png'), 
+                   '/static': static_dir, 
                    '/repos': 'repos', 
-                   settings.SITE_URL.rstrip('/'): settings.SITE_PATH}
+                   settings.DEV_SITE_URL.rstrip('/'): settings.SITE_PATH}
     return SiteServeMiddleware(application, static_files, fallback_mimetype='text/html')
 
 def make_cgi_app():
     """
     Generates the CGI application, with CGI we can't afford to serve static files with
-    the application. That has to be done by the real server, eg. apache: see the .htaccess
+    werkzeug; that has to be done by the real server, eg. apache: see the cgi.htaccess
     file in the site file (the one with settings.py).
     """
     return application
@@ -54,26 +54,26 @@ def application(request):
     return response
 
 urls = (
-    ('logout$', 'logout'),
-    ('add-repo', 'edit_repo'),
-    ('view-repo-(.+)$', 'view_repo'),
-    ('add-page$', 'edit_page'),
-    ('edit-page-last$', 'edit_last_page'),
-    ('edit-page-(.+)$', 'edit_page'),
-    ('add-template$', 'edit_template'),
-    ('edit-template-(.+)$', 'edit_template'),
-    ('add-static$', 'edit_static'),
-    ('edit-static-(.+)$', 'edit_static'),
-    ('add-libfile', 'edit_libfile'),
-    ('edit-libfile-(.+)$', 'edit_libfile'),
-    ('add-globcon', 'edit_globcon'),
-    ('edit-globcon-(.+)$', 'edit_globcon'),
-    ('submit.json', 'json_response'),
-    ('add-user$', 'edit_user'),
-    ('user$', 'edit_this_user'),
-    ('edit-user-last$', 'edit_last_user'),
-    ('edit-user-(.+)$', 'edit_user'),
-    ('set-password-(.+)$', 'set_user_password'),
+    ('/logout$', 'logout'),
+    ('/add-repo', 'edit_repo'),
+    ('/view-repo-(.+)$', 'view_repo'),
+    ('/add-page$', 'edit_page'),
+    ('/edit-page-last$', 'edit_last_page'),
+    ('/edit-page-(.+)$', 'edit_page'),
+    ('/add-template$', 'edit_template'),
+    ('/edit-template-(.+)$', 'edit_template'),
+    ('/add-static$', 'edit_static'),
+    ('/edit-static-(.+)$', 'edit_static'),
+    ('/add-libfile', 'edit_libfile'),
+    ('/edit-libfile-(.+)$', 'edit_libfile'),
+    ('/add-globcon', 'edit_globcon'),
+    ('/edit-globcon-(.+)$', 'edit_globcon'),
+    ('/submit.json', 'json_response'),
+    ('/add-user$', 'edit_user'),
+    ('/edit-user-last$', 'edit_last_user'),
+    ('/edit-user-(.+)$', 'edit_user'),
+    ('/set-password-(.+)$', 'set_user_password'),
+    ('/user$', 'edit_this_user'),
 )
 
 class View(object):
@@ -94,7 +94,6 @@ class View(object):
                 self._add_msg(self.request.login_message, ('errors', 'success')[self.request.valid_user])
             self.isadmin = self.request.valid_user and self.request.user and self.request.user['admin']
             self._env = jinja2.Environment(loader=jinja2.FileSystemLoader(EDITOR_TEMPLATE_DIR))
-            self.site_url = (settings.SITE_URL, '/')[settings.SITE_URL == '']
             if SERVER_MODE == SERVER_MODES.CGI:
                 self._uri = request.environ['REQUEST_URI']
                 sim = _string_similar_point(self._uri, request.environ['SCRIPT_NAME'])
@@ -102,7 +101,7 @@ class View(object):
             else:
                 url = request.base_url.replace('http://', '')
                 self._uri = str(url[url.index('/'):])
-            self.edit_static_url = join_uri(self.site_edit_url, 'static/')
+            self.edit_static_url = ds.join_uri(self.site_edit_url, 'static/')
             self._process_forms()
             self._generate_page(self._uri, loggedin = request.valid_user, error = self.processing_error)
                   
@@ -128,14 +127,13 @@ class View(object):
         self.processing_error = None
         try:
             if self.request.valid_user:
-                proc_form = ProcessForm(self._add_msg, self.request, self.isadmin, self.request.username)
+                proc_form = ProcessForm(self)
                 if proc_form.regen_users:
-                    self.logout(None)
+                    self.request.regenerate_users()
                 self.created_item = proc_form.created_item
             else:
                 AnonFormProcessor(self._add_msg, self.request.form)
         except Exception, e:
-            print 'processing _process_forms error:', e
             _, _, exc_traceback = sys.exc_info()
             self.processing_error = (e, exc_traceback)
         
@@ -145,7 +143,7 @@ class View(object):
                         'site_title': '%s Editor' % settings.SITE_NAME, 
                         'static_url': self.edit_static_url, 
                         'edit_uri': self.site_edit_url,
-                        'site_url': self.site_url, 
+                        'site_url': settings.DEV_SITE_URL, 
                         'json_submit_url': '%ssubmit.json' % self.site_edit_url, 
                         'version': DonkeySimple.__version__
                         }
@@ -167,7 +165,7 @@ class View(object):
                 self.login()
         else:
             found = False
-            for reg, func in urls:  
+            for reg, func in urls:
                 m = re.search(reg, uri)
                 if m:
                     found = True
@@ -354,7 +352,7 @@ class View(object):
             self.context['file_id'] = cfile.id
             self.context['active_repo'] = cfile.repo
             self.context['file_type'] = static.get_file_type(self.context['file_name'])
-            static_uri = join_uri(self.site_edit_url, ds.REPOS_DIR, cfile.repo, static.DIR, cfile.name)
+            static_uri = ds.join_uri(self.site_edit_url, ds.REPOS_DIR, cfile.repo, static.DIR, cfile.name)
             if self.context['file_type'] == 'Text':
                 self.context['file_text'] = escape(content)
                 self.context['show_file_text'] = True
@@ -414,6 +412,8 @@ class View(object):
                 return self._permission_denied()
         self.context['action_uri'] = self.site_edit_url + 'edit-user-last'
         if uid is not None:
+            if uid not in self.request.users:
+                return self._error_page('user "%s" not found' % uid)
             self.context['existing_user'] = True
             self.context['password_uri'] = self.site_edit_url + 'set-password-%s' % uid
             edit_user = self.request.users[uid]
@@ -431,8 +431,8 @@ class View(object):
         self._template = self._env.get_template('edit_user.html')
         
     def set_user_password(self, uid):
-        edit_username = self.request.users[uid]
-        if not self.isadmin and edit_username != self.request.username or self.request.is_anon:
+        edit_username = uid
+        if (not self.isadmin and edit_username != self.request.username) or self.request.is_anon:
             return self._permission_denied()
         self.context['edit_username'] = uid
         self._template = self._env.get_template('set_password.html')
@@ -446,10 +446,13 @@ class View(object):
     def _error_page(self, e, code = httplib.INTERNAL_SERVER_ERROR, tb = None):
         error_details = None
         if settings.DEBUG:
+            if issubclass(type(e), Exception):
+                error_details = '%s: %s' % (e.__class__.__name__, str(e))
             if tb:
-                error_details = '\n'.join(traceback.format_tb(tb))
+                print '\n'.join(traceback.format_tb(tb))
             else:
-                error_details = traceback.format_exc()
+                exc = traceback.format_exc()
+                if exc: print exc
         self._bad(str(e),error_details=error_details, code=code)
         
     def _bad(self, error_name, error_details = None, code = httplib.BAD_REQUEST):
@@ -458,7 +461,7 @@ class View(object):
         self.response_code = code
         self.context['error_name'] = error_name
         if error_details:
-#             self.context['error_details'] = str(error_details)
+            self.context['error_details'] = str(error_details)
             traceback.print_exc()
         if 'submit.json' in self._uri:
             self._json_response(self.context)
@@ -471,14 +474,30 @@ class View(object):
         else:
             self._msgs[mtype].append(msg)
 
+
+extra_page_content = """
+<a href="/" style="
+position: fixed;
+left: 50px;
+bottom: 50px;
+padding: 10px 20px;
+background-color: #2d6ca2;
+color: white !important;
+border-radius: 5px;
+text-decoration: none !important;
+background-image: linear-gradient(to bottom,#428bca 0,#2d6ca2 100%);
+border: 1px solid #2b669a;
+">Back to Donkey Simple</a>\n
+"""
+
 class SiteServeMiddleware(SharedDataMiddleware):
     
     def get_directory_loader(self, directory):        
         def loader(path):
             """
             changed from original to serve index.html if the directory is requested
-            and /*.html if if /* is requested.
-            """ 
+            and /*.html if /* is requested.
+            """
             path_orig = path
             if path is not None:
                 path = os.path.join(directory, path)
@@ -499,6 +518,20 @@ class SiteServeMiddleware(SharedDataMiddleware):
                     return attempt_fname, self._opener(filepath)
             return None, None
         return loader
+    
+        
+    def _opener(self, filename):
+        if filename.endswith('.html'):
+            new_filename = os.path.basename(filename) .replace('.html', '.tmp.html')
+            new_path = os.path.join(settings.SITE_PATH_TMP, new_filename)
+            if not os.path.exists(new_path):
+                text = open(filename, 'r').read()
+                text = text.replace('</body>', extra_page_content + '</body>')
+                if not os.path.exists(settings.SITE_PATH_TMP):
+                    os.mkdir(settings.SITE_PATH_TMP)
+                open(new_path, 'w').write(text)
+            filename = new_path
+        return super(SiteServeMiddleware, self)._opener(filename)
 
 
 class CatchStdout(object):
@@ -550,7 +583,7 @@ class CatchStdout(object):
         if self.stdout_text.strip('\r\t\n ') == '':
             return ''
         debug  = '==========DEBUG OUTPUT==========\n'
-        debug += '%s\n' % self.stdout_text
+        debug += '%s\n' % self.stdout_text.strip('\r\t\n ')
         debug += '================================'
         return debug
  
@@ -564,14 +597,6 @@ def get_font_name(filename, add_msg):
         print e_str
         add_msg(e_str, 'errors')
         return 'unknown'
-
-def join_uri(*args):
-    uri = '/' + '/'.join(p.strip('/') for p in args)
-    if args[-1].endswith('/'):
-        uri += '/'
-    uri = uri.replace('//', '/')
-    return uri
-
     
 def _string_similar_point(s1, s2):
     i = 0
